﻿-- pgCon2014 PostgreSQL Working with Network Data
-- James Hanson 21-May-14
-- MyIP known bad IPs and web crawlers
-- Downloaded from http://myip.ms/files/blacklist/general/full_blacklist_database.zip and 
--                 http://myip.ms/downloads/web_bots/myip.ms-webcrawlers.zip
-- NOTE:  This data set includes the country code, which we could join with the Airport data.

-- COPY into temporary table for processing
-- Create temporary table
CREATE TABLE myip_temp (
   id SERIAL,
   col1 TEXT);

-- Load temporary table
-- NOTE:  ran # dos2unix full_blacklist_database.txt
COPY myip_temp(  
  col1)
 FROM '/Pgcon14/Scripts/070_MyIP/full_blacklist_database.txt'
 WITH DELIMITER '|'  -- use a character that is not in the dataset
 CSV;

-- TRUNCATE TABLE myip_temp;
-- DROP TABLE myip_temp;
-- SELECT * FROM myip_temp LIMIT 100;
-- SELECT to_char(count(*), '999G999') from myip_temp;

-- remove header and footer rows
-- identify header
SELECT *
FROM myip_temp
ORDER BY id;

-- remove 15 header rows
DELETE FROM myip_temp
WHERE id = any(array(
  SELECT id
  FROM myip_temp
  ORDER BY id
  LIMIT 15)
  );

-- identify footer
SELECT *
FROM myip_temp
ORDER BY id DESC;

-- Pull the fields we want
WITH myip_array AS (
SELECT 
  regexp_split_to_array(col1, '\,') AS col1
FROM myip_temp
LIMIT 1000
)
SELECT 
  TRIM(regexp_replace(LEFT(col1[1], -12), E'\t+', ''))::inet AS ipAddress,
  RIGHT(col1[1], 10)::date AS date,
--  col1[1] AS col1_1,  -- contains both IP and date plus \t.  Separated by #.  The last 11 characters are date
  TRIM(col1[2])::VARCHAR(200) AS host,
  TRIM(col1[3])::CHAR(3) AS countryID,
  TRIM(col1[4])::int AS blacklistID
--  col1[5] AS col1_5
FROM myip_array;

-- Create and populate us_ipgeo from us_ipgeo_temp
-- Try it once with a LIMIT statement
SELECT * 
INTO myip
FROM (
WITH myip_array AS (
SELECT 
  regexp_split_to_array(col1, '\,') AS col1
FROM myip_temp
-- LIMIT 1000
)
SELECT 
  TRIM(regexp_replace(LEFT(col1[1], -12), E'\t+', ''))::inet AS ipAddress,
  RIGHT(col1[1], 10)::date AS date,
--  col1[1] AS col1_1,  -- contains both IP and date plus \t.  Separated by #.  The last 11 characters are date
  TRIM(col1[2])::VARCHAR(200) AS host,
  TRIM(col1[3])::CHAR(3) AS countryID,
  TRIM(col1[4])::int AS blacklistID
--  col1[5] AS col1_5
FROM myip_array
) AS tmp_table;

COMMENT ON TABLE myip IS 'Downloaded from http://myip.ms/files/blacklist/general/full_blacklist_database.zip';

-- TRUNCATE TABLE myip;
-- DROP TABLE myip;
-- SELECT * FROM myip ORDER BY ipaddress DESC LIMIT 100;
-- SELECT to_char(count(*), '999G999') FROM myip;

-- Now load malicious Web crawler data
CREATE TABLE myip_web_temp (
   id SERIAL,
   col1 TEXT);

-- Load temporary table
-- NOTE:  ran # dos2unix myip.ms-webcrawlers.txt
COPY myip_web_temp(  
  col1)
 FROM '/Pgcon14/Scripts/070_MyIP/myip.ms-webcrawlers.txt'
 WITH DELIMITER '|'  -- use a character that is not in the dataset
---  CSV
 ;

-- TRUNCATE TABLE myip_web_temp;
-- DROP TABLE myip_web_temp;
-- SELECT * FROM myip_web_temp LIMIT 100; 
-- SELECT to_char(count(*), '999G999') FROM myip_web_temp;

-- remove header and footer rows
-- identify header
SELECT *
FROM myip_web_temp
ORDER BY id;

-- remove 14 header rows
DELETE FROM myip_web_temp
WHERE id = any(array(
  SELECT id
  FROM myip_web_temp
  ORDER BY id
  LIMIT 14)
  );

-- identify footer
-- Take a look at the %&!@'d rows that begin with #
SELECT *
FROM myip_web_temp
ORDER BY id DESC;

-- This file contains intersperced comments that begin with '#'
SELECT *
FROM myip_web_temp
WHERE LEFT(col1, 1) != '#'
  AND col1 != ''
ORDER BY id DESC;

-- Pull the fields we want
SELECT *
INTO myip_web
FROM (
SELECT DISTINCT
  col1::inet AS webbot_ip
FROM myip_web_temp
WHERE LEFT(col1, 1) != '#'
  AND col1 != '') AS temp;

COMMENT ON TABLE myip_web IS 'Downloaded from http://myip.ms/downloads/web_bots/myip.ms-webcrawlers.zip';

-- TRUNCATE TABLE myip_web;
-- DROP TABLE myip_web;
-- SELECT * FROM myip_web LIMIT 100; 
-- SELECT to_char(count(*), '99G999') FROM myip_web;

-- cleanup
DROP TABLE myip_temp;
DROP TABLE myip_web_temp;