﻿-- pgCon2014 PostgreSQL Working with Network Data
-- James Hanson 21-May-14
-- Dragon Research Group HTTP Scanner Report
-- Downloaded from shttp://www.dragonresearchgroup.org/insight/http-report.txt


-- COPY into temporary table for processing because file contains header and footer lines
--    that the COPY command cannot handle
-- Create temporary table
CREATE TABLE http_report_temp (
   id SERIAL,
   col1 TEXT);

-- Load temporary table
COPY http_report_temp(  
  col1)
 FROM '/Pgcon14/Scripts/040_DRG_IPs/http-report.txt'
 WITH DELIMITER '$'   -- use a character that is not in the dataset
 CSV;

-- TRUNCATE TABLE http_report_temp;
-- DROP TABLE http_report_temp;
-- SELECT * FROM http_report_temp LIMIT 100;
-- SELECT to_char(count(*), '9G999') FROM http_report_temp;

-- remove header and footer rows
-- identify header
SELECT *
FROM http_report_temp
ORDER BY id
LIMIT 100;

-- remove 78 header rows
DELETE FROM http_report_temp
WHERE id = any(array(
  SELECT id
  FROM http_report_temp
  ORDER BY id
  LIMIT 78)
  );

-- remove footer
SELECT *
FROM http_report_temp
ORDER BY id DESC
LIMIT 100;

-- remove 4 footer rows
DELETE FROM http_report_temp
WHERE id = any(array(
  SELECT id
  FROM http_report_temp
  ORDER BY id DESC
  LIMIT 4)
  );
  
-- Use regexp_split_to_array to look at the data
WITH http_report_array AS (
SELECT regexp_split_to_array(col1, '\|') AS col1
FROM http_report_temp
LIMIT 10
)
SELECT 
  col1[1] AS col1_1,
  col1[2] AS col1_2,
  col1[3] AS col1_3,
  col1[4] AS col1_4,
  col1[5] AS col1_5,
  col1[6] AS col1_6  -- Verify that the last array element is NULL
FROM http_report_array;

-- Manipulate and CAST the fields as needed
WITH http_report_array AS (
SELECT regexp_split_to_array(col1, '\|') AS col1
FROM http_report_temp
LIMIT 10
)
SELECT 
  col1[1]::int AS asn,  -- ASN = Autonomous System Number
  TRIM(col1[2])::varchar(30) AS asname,  -- file header listed field size
  TRIM(col1[3])::inet AS netblock,    -- PG has operators to tell if an IP is in this subnet
  col1[4]::timestamp with time zone AS utc_time,
  TRIM(col1[5])::varchar(10) AS category
  -- col1[6] AS col1_6
FROM http_report_array;

-- Create and populate http_report from http_report_temp
-- Try it once with a LIMIT statement
SELECT * 
INTO http_report
FROM (
WITH http_report_array AS (
SELECT regexp_split_to_array(col1, '\|') AS col1
FROM http_report_temp
-- LIMIT 10
)
SELECT 
  col1[1]::int AS asn,
  TRIM(col1[2])::varchar(30) AS asname,  -- file header listed field size
  TRIM(col1[3])::inet AS netblock,
  col1[4]::timestamp with time zone AS utc_time,
  TRIM(col1[5])::varchar(10) AS category
  -- col1[6] AS col1_6
FROM http_report_array
) AS tmp_table;

-- TRUNCATE TABLE http_report;
-- DROP TABLE http_report;
-- SELECT * FROM http_report LIMIT 100;
-- SELECT to_char(count(*), '9G999') FROM http_report;

-- cleanup
DROP TABLE http_report_temp;