﻿-- pgCon2014 PostgreSQL Working with Network Data
-- James Hanson 21-May-14
-- Top 500 hosts
-- Downloaded from http://myip.ms/download/reports/2/previewName/report.xls
-- NOTE:  I saved the file as 20140202_report_3_300_000_world_websites_hosting_information_february_2014--downloaded_from_myip_ms.txt

-- COPY into temporary table for processing
-- Create temporary table
CREATE TABLE tophosts_temp (
   id SERIAL,
   col1 TEXT);

-- If you don't change the client encoding, this will error with "invalid byte sequence for encoding "UTF8""
-- Found the solution on "http://stackoverflow.com/questions/4867272/invalid-byte-sequence-for-encoding-utf8"
-- There is no harm in trying it ... nothing will break!
SET client_encoding TO 'latin1';

-- Load temporary table
COPY tophosts_temp(  
  col1)
 FROM '/Pgcon14/Scripts/070_MyIP/20140202_report_3_300_000_world_websites_hosting_information_february_2014--downloaded_from_myip_ms.txt'
 WITH DELIMITER '|'  -- use a character that is not in the dataset
 CSV;

-- TRUNCATE TABLE tophosts_temp;
-- DROP TABLE tophosts_temp;
-- SELECT * FROM tophosts_temp LIMIT 100;

-- remove header and footer rows
-- identify header
SELECT *
FROM tophosts_temp
ORDER BY id;

-- remove 6 header rows
DELETE FROM tophosts_temp
WHERE id = any(array(
  SELECT id
  FROM tophosts_temp
  ORDER BY id
  LIMIT 6)
  );

-- identify footer
SELECT *
FROM tophosts_temp
ORDER BY id DESC;

-- remove 5 footer rows
DELETE FROM tophosts_temp
WHERE id = any(array(
  SELECT id
  FROM tophosts_temp
  ORDER BY id DESC
  LIMIT 5)
  );

-- Pull the fields we want
WITH tophosts_array AS (
SELECT 
  regexp_split_to_array(col1, '\t') AS col1
FROM tophosts_temp
LIMIT 1000
)
SELECT 
  col1[1] AS col1_1,
  col1[2] AS col1_2,
  col1[3] AS col1_3,
  col1[4] AS col1_4,
  col1[5] AS col1_5,
  col1[6] AS col1_6,
  col1[7] AS col1_7,
  col1[8] AS col1_8,
  col1[9] AS col1_9,
  col1[10] AS col1_10,
  col1[11] AS col1_11,
  col1[12] AS col1_12,
  col1[13] AS col1_13,
  col1[14] AS col1_14,
  col1[15] AS col1_15,
  col1[16] AS col1_16,
  col1[17] AS col1_17,
  col1[18] AS col1_18,
  col1[19] AS col1_19,
  col1[20] AS col1_20,
  col1[21] AS col1_21,
  col1[22] AS col1_22,
  col1[23] AS col1_23,
  col1[24] AS col1_24,
  col1[25] AS col1_25,
  col1[26] AS col1_26
FROM tophosts_array;

-- Manipulate and CAST the fields as needed
-- got the column definitions from a Excel file
WITH tophosts_array AS (
SELECT 
  regexp_split_to_array(col1, '\t') AS col1
FROM tophosts_temp
LIMIT 1000
)
SELECT 
  col1[1]::INT AS WW_site_rank,
  col1[2]::VARCHAR(200) AS website,
  col1[3]::inet AS website_ip,
  col1[4]::VARCHAR(100) AS hosting_country,
  col1[5]::VARCHAR(100) AS hosting_company,
  LEFT(col1[6], (POSITION('-' IN col1[6])-2))::inet AS owner_ip_from,
  RIGHT(col1[6], (-1 * POSITION('-' IN col1[6])) - 1)::inet AS owner_ip_to,
  col1[7]::VARCHAR(100) AS owner_country,
  col1[8]::VARCHAR(150) AS parent_hosting_company,
  col1[9]::VARCHAR(100) AS parent_ip_range,
  col1[10]::VARCHAR(100) AS parent_country,
  col1[11]::VARCHAR(100) AS top_parent_hosting_company,
  col1[12]::VARCHAR(100) AS top_parent_ip_range,
  col1[13]::VARCHAR(100) AS top_parent_country,
  col1[14]::VARCHAR(100) AS nameserver1,
  NULLIF(col1[15], '')::inet AS nameserver1_ip,
  col1[16]::VARCHAR(100) AS nameserver2,
  NULLIF(col1[17], '')::inet AS nameserver2_ip,
  col1[18]::VARCHAR(100) AS nameserver3,
  NULLIF(col1[19], '')::inet AS nameserver3_ip,
  col1[20]::VARCHAR(100) AS nameserver4,
  NULLIF(col1[21], '')::inet AS nameserver4_ip,
  col1[22]::VARCHAR(100) AS nameserver5,
  NULLIF(col1[23], '')::inet AS nameserver5_ip,
  col1[24]::VARCHAR(100) AS nameserver5,
  NULLIF(col1[25], '')::inet AS nameserver6_ip
--  col1[26] AS col1_26
FROM tophosts_array;

-- Create and populate tophosts from tophosts_array
-- Try it once with a LIMIT statement
SELECT *
INTO tophosts
FROM (
WITH tophosts_array AS (
SELECT 
  regexp_split_to_array(col1, '\t') AS col1
FROM tophosts_temp
LIMIT 1000
)
SELECT 
  col1[1]::INT AS WW_site_rank,
  col1[2]::VARCHAR(200) AS website,
  col1[3]::inet AS website_ip,
  col1[4]::VARCHAR(100) AS hosting_country,
  col1[5]::VARCHAR(100) AS hosting_company,
  LEFT(col1[6], (POSITION('-' IN col1[6])-2))::inet AS owner_ip_from,
  RIGHT(col1[6], (-1 * POSITION('-' IN col1[6])) - 1)::inet AS owner_ip_to,
  col1[7]::VARCHAR(100) AS owner_country,
  col1[8]::VARCHAR(150) AS parent_hosting_company,
  col1[9]::VARCHAR(100) AS parent_ip_range,
  col1[10]::VARCHAR(100) AS parent_country,
  col1[11]::VARCHAR(100) AS top_parent_hosting_company,
  col1[12]::VARCHAR(100) AS top_parent_ip_range,
  col1[13]::VARCHAR(100) AS top_parent_country,
  col1[14]::VARCHAR(100) AS nameserver1,
  NULLIF(col1[15], '')::inet AS nameserver1_ip,
  col1[16]::VARCHAR(100) AS nameserver2,
  NULLIF(col1[17], '')::inet AS nameserver2_ip,
  col1[18]::VARCHAR(100) AS nameserver3,
  NULLIF(col1[19], '')::inet AS nameserver3_ip,
  col1[20]::VARCHAR(100) AS nameserver4,
  NULLIF(col1[21], '')::inet AS nameserver4_ip,
  col1[22]::VARCHAR(100) AS nameserver5,
  NULLIF(col1[23], '')::inet AS nameserver5_ip,
  col1[24]::VARCHAR(100) AS nameserver6,
  NULLIF(col1[25], '')::inet AS nameserver6_ip
--  col1[26] AS col1_26
FROM tophosts_array
) AS tmp_table;

COMMENT ON TABLE tophosts IS 'Downloaded from http://myip.ms/download/reports/2/previewName/report.xls';

-- TRUNCATE TABLE tophosts;
-- DROP TABLE tophosts;
-- SELECT * FROM tophosts LIMIT 100;
-- SELECT count(*) FROM tophosts;

-- Cleanup
DROP TABLE tophosts_temp;
SET client_encoding TO 'UTF-8';