﻿-- pgCon2014 PostgreSQL Working with Network Data
-- James Hanson 21-May-14
-- Dragon Research Group sshpwauth IP
-- IPs that are known to have originated brute-force ssh password attacks
-- Downloaded from http://www.dragonresearchgroup.org/insight/sshpwauth.txt

-- COPY into temporary table for processing because file contains header and footer lines
--    that the COPY command cannot handle
-- Create temporary table
CREATE TABLE sshpwauth_temp (
   id SERIAL,
   col1 TEXT);

-- Load temporary table
COPY sshpwauth_temp(  
  col1)
 FROM '/Pgcon14/Scripts/040_DRG_IPs/sshpwauth.txt'
 WITH DELIMITER '$'   -- use a character that is not in the dataset
 CSV;

-- TRUNCATE TABLE sshpwauth_temp;
-- DROP TABLE sshpwauth_temp;
-- SELECT * FROM sshpwauth_temp LIMIT 100;
-- SELECT count(*) FROM sshpwauth_temp;

-- remove header and footer rows
-- identify header
SELECT *
FROM sshpwauth_temp
ORDER BY id
LIMIT 100;

-- remove 74 header rows
DELETE FROM sshpwauth_temp
WHERE id = any(array(
  SELECT id
  FROM sshpwauth_temp
  ORDER BY id
  LIMIT 74)
  );

-- identify footer
SELECT *
FROM sshpwauth_temp
ORDER BY id DESC
LIMIT 100;

-- remove 4 footer rows
DELETE FROM sshpwauth_temp
WHERE id = any(array(
  SELECT id
  FROM sshpwauth_temp
  ORDER BY id DESC
  LIMIT 4)
  );

-- Use regexp_split_to_array to look at the data
WITH sshpwauth_array AS (
SELECT regexp_split_to_array(col1, '\|') AS col1
FROM sshpwauth_temp
LIMIT 10
)
SELECT 
  col1[1] AS col1_1,
  col1[2] AS col1_2,
  col1[3] AS col1_3,
  col1[4] AS col1_4,
  col1[5] AS col1_5,
  col1[6] AS col1_6
FROM sshpwauth_array;

-- Manipulate and CAST the fields as needed
WITH sshpwauth_array AS (
SELECT regexp_split_to_array(col1, '\|') AS col1
FROM sshpwauth_temp
LIMIT 10
)
SELECT 
  col1[1]::int AS asn,
  TRIM(col1[2])::varchar(30) AS asname,  -- file header listed field size
  TRIM(col1[3])::inet AS netblock,
  col1[4]::timestamp with time zone AS utc_time,
  TRIM(col1[5])::varchar(10) AS category
  -- col1[6] AS col1_6
FROM sshpwauth_array;

SELECT * 
INTO sshpwauth
FROM (
WITH sshpwauth_array AS (
SELECT regexp_split_to_array(col1, '\|') AS col1
FROM sshpwauth_temp
-- LIMIT 10
)
SELECT 
  col1[1]::int AS asn,
  TRIM(col1[2])::varchar(30) AS asname,  -- file header listed field size
  TRIM(col1[3])::inet AS netblock,
  col1[4]::timestamp with time zone AS utc_time,
  TRIM(col1[5])::varchar(10) AS category
  -- col1[6] AS col1_6
FROM sshpwauth_array
) AS tmp_table;

COMMENT ON TABLE sshpwauth IS 'Downloaded from shttp://www.dragonresearchgroup.org/insight/sshpwauth.txt';

-- TRUNCATE TABLE sshpwauth;
-- DROP TABLE sshpwauth;
-- SELECT * FROM sshpwauth LIMIT 100;
-- SELECT count(*) FROM sshpwauth;

-- cleanup
DROP TABLE sshpwauth_temp;