﻿-- PgCon14 Working with Network Data
-- James Hanson
-- 070_MyIP data
-- Top 500 Hosts restructure
-- Create tophosts table using and array of ip ranges

-- Look at our data
SELECT *
FROM tophosts;

-- Want to use a range for the IPs
-- Create custom type
CREATE TYPE inetrange AS RANGE (
  SUBTYPE = inet
);

-- DROP TYPE inetrange;

-- Check out our data
-- SELECT inclusive, [], range for our inetrange
SELECT
  website,
  ww_site_rank,
  hosting_country,
  hosting_company,
  owner_country,
  parent_hosting_company,
  parent_country,
  inetrange(owner_ip_from, owner_ip_to, '[]') AS owner_range,  -- [] means upper and lower inclusive
  parent_ip_range
FROM tophosts
ORDER BY owner_range;

-- Check out the parent_ip
-- Can we make that a range too?
WITH parent_ip_range AS (
SELECT 
  website,
  ww_site_rank,
--  hosting_country,
--  hosting_company,
--  owner_country,
--  parent_hosting_company,
--  parent_country,
  inetrange(owner_ip_from, owner_ip_to, '[]') AS owner_range,
  regexp_split_to_array(parent_ip_range, ' ') pir
FROM tophosts)
SELECT 
  website,
  ww_site_rank,
--  hosting_country,
--  hosting_company,
--  owner_country,
--  parent_hosting_company,
--  parent_country,
  owner_range,
  inetrange(NULLIF(pir[1], '')::inet, pir[3]::inet, '[]') AS parent_ip_range,  -- don't like how NULLs are handled
  NULLIF(inetrange(NULLIF(pir[1], '')::inet, pir[3]::inet, '[]'), '(,)') AS parent_ip_range
FROM parent_ip_range;

-- What about an inetrange array?
CREATE TABLE tophosts_range_temp AS 
SELECT 
 *
FROM (
WITH parent_ip_range AS (
SELECT 
  website,
  ww_site_rank,
  hosting_country,
  hosting_company,
  owner_country,
  parent_hosting_company,
  parent_country,
  inetrange(owner_ip_from, owner_ip_to, '[]') AS owner_range,
  regexp_split_to_array(parent_ip_range, ' ') pir
FROM tophosts)
SELECT 
  website,
  ww_site_rank,
  hosting_country,
  hosting_company,
  owner_country,
  parent_hosting_company,
  parent_country,
  owner_range,
  NULLIF(inetrange(NULLIF(pir[1], '')::inet, pir[3]::inet, '[]'), '(,)') AS parent_ip_range
FROM parent_ip_range
) AS temp;

-- DROP TABLE tophosts_range_temp;
-- TRUNCATE TABLE tophosts_range_temp;
-- SELECT * FROM tophosts_range_temp;

-- check out our syntax with a SELECT
SELECT 
  website,
  CASE WHEN parent_ip_range IS NULL
    THEN array[owner_range]
    ELSE array[owner_range,  parent_ip_range]
  END AS ip_range,
  ww_site_rank,
  hosting_country,
  hosting_company,
  owner_country,
  parent_hosting_company,
  parent_country
FROM tophosts_range_temp;

-- create table tophosts_range
CREATE TABLE tophosts_range AS
SELECT *
FROM (
SELECT 
  website,
  CASE WHEN parent_ip_range IS NULL
    THEN array[owner_range]
    ELSE array[owner_range,  parent_ip_range]
  END AS ip_range,
  ww_site_rank,
  hosting_country,
  hosting_company,
  owner_country,
  parent_hosting_company,
  parent_country
FROM tophosts_range_temp) AS temp;

-- SELECT * FROM tophosts_range;
-- TRUNCATE TABLE tophosts_range;
-- DROP TABLE tophosts_range;

-- cleanup
DROP TABLE tophosts_range_temp;