﻿-- pgCon2014 PostgreSQL Working with Network Data
-- James Hanson 21-May-14
-- Dragon Research Group vnc probe IPs
-- IP addresses known to have originated vnc probes
-- Downloaded from shttp://www.dragonresearchgroup.org/insight/vncprobe.txt

-- COPY into temporary table for processing because file contains header and footer lines
--    that the COPY command cannot handle
-- Create temporary table
CREATE TABLE vncprobe_temp (
   id SERIAL,
   col1 TEXT);

-- Load temporary table
COPY vncprobe_temp(  
  col1)
 FROM '/Pgcon14/Scripts/040_DRG_IPs/vncprobe.txt'
 WITH DELIMITER '$'   -- use a character that is not in the dataset
 CSV;

-- TRUNCATE TABLE vncprobe_temp;
-- DROP TABLE vncprobe_temp;
-- SELECT * FROM vncprobe_temp LIMIT 100;
-- SELECT count(*) FROM vncprobe_temp;

-- remove header and footer rows
-- identify header
SELECT *
FROM vncprobe_temp
ORDER BY id
LIMIT 100;

-- remove 74 header rows
DELETE FROM vncprobe_temp
WHERE id = any(array(
  SELECT id
  FROM vncprobe_temp
  ORDER BY id
  LIMIT 74)
  );

-- identify footer
SELECT *
FROM vncprobe_temp
ORDER BY id DESC
LIMIT 100;

-- remove 5 footer rows
DELETE FROM vncprobe_temp
WHERE id = any(array(
  SELECT id
  FROM vncprobe_temp
  ORDER BY id DESC
  LIMIT 5)
  );

-- Use regexp_split_to_array to look at the data
WITH vncprobe_array AS (
SELECT regexp_split_to_array(col1, '\|') AS col1
FROM vncprobe_temp
LIMIT 10
)
SELECT 
  col1[1] AS col1_1,
  col1[2] AS col1_2,
  col1[3] AS col1_3,
  col1[4] AS col1_4,
  col1[5] AS col1_5,
  col1[6] AS col1_6
FROM vncprobe_array;

-- Manipulate and CAST the fields as needed
WITH vncprobe_array AS (
SELECT regexp_split_to_array(col1, '\|') AS col1
FROM vncprobe_temp
LIMIT 1000
)
SELECT 
  col1[1]::int AS asn,
  TRIM(col1[2])::varchar(30) AS asname,  -- file header listed field size
  TRIM(col1[3])::inet AS netblock,
  col1[4]::timestamp with time zone AS utc_time,
  TRIM(col1[5])::varchar(10) AS category
  -- col1[6] AS col1_6
FROM vncprobe_array;

SELECT * 
INTO vncprobe
FROM (
WITH vncprobe_array AS (
SELECT regexp_split_to_array(col1, '\|') AS col1
FROM vncprobe_temp
-- LIMIT 10
)
SELECT 
  col1[1]::int AS asn,
  TRIM(col1[2])::varchar(30) AS asname,  -- file header listed field size
  TRIM(col1[3])::inet AS netblock,
  col1[4]::timestamp with time zone AS utc_time,
  TRIM(col1[5])::varchar(10) AS category
  -- col1[6] AS col1_6
FROM vncprobe_array
) AS tmp_table;

COMMENT ON TABLE vncprobe IS 'Downloaded from shttp://www.dragonresearchgroup.org/insight/vncprobe.txt';

-- TRUNCATE TABLE vncprobe;
-- DROP TABLE vncprobe;
-- SELECT * FROM vncprobe LIMIT 100;
-- SELECT count(*) FROM vncprobe;

-- cleanup
DROP TABLE vncprobe_temp;