﻿-- pgCon2014 Working with Netowork Data
-- James Hanson 21-May-14
-- Airport data DDL statements

-- Countries
CREATE TABLE countries (
  id BIGINT PRIMARY KEY,
  code CHAR(10) NOT NULL UNIQUE,
  name VARCHAR(100),
  continent CHAR(4),
  wikipedia_link VARCHAR(500),
  keywords VARCHAR(300));
  
COMMENT ON TABLE countries IS 'Data from http://ourairports.com/data/';

COPY countries
FROM '/Pgcon14/Scripts/020_Airports/countries.csv' WITH DELIMITER ',' CSV HEADER;
 
-- TRUNCATE TABLE countries;
-- DROP TABLE countries;
-- SELECT * FROM countries LIMIT 10;
-- SELECT count(*) FROM countries;

-- Airports
CREATE TABLE airports (
  id BIGINT PRIMARY KEY,
  ident CHAR(10) NOT NULL UNIQUE,
  type VARCHAR(50) NOT NULL,
  name VARCHAR(100),
  latitude_deg REAL NOT NULL,
  longitude_deg REAL NOT NULL,
  location GEOMETRY,  		-- Note the GEOMETRY type.  GEOGRAPHY has some advantages, but is more complex
  elevation_ft REAL,
  continent CHAR(4),
  iso_country CHAR(10) REFERENCES countries(code),
  iso_region CHAR(10),
  municipality VARCHAR(100),
  scheduled_service BOOLEAN,
  gps_code CHAR(10),
  iata_code CHAR(10),
  local_code CHAR(10),
  home_link VARCHAR(500),
  wikipedia_link VARCHAR(500),
  keywords VARCHAR(300));
  
COMMENT ON TABLE airports IS 'Data from http://ourairports.com/data/';

COPY airports(
  id,
  ident,
  type,name,
  latitude_deg,
  longitude_deg,
  elevation_ft,
  continent,
  iso_country,
  iso_region,
  municipality,
  scheduled_service,
  gps_code,
  iata_code,
  local_code,
  home_link,
  wikipedia_link,keywords
  )
  FROM '/Pgcon14/Scripts/020_Airports/airports.csv' WITH DELIMITER ',' CSV HEADER;

SELECT * FROM airports LIMIT 10; -- NOTE that the GEOMETRY field is NULL

-- Update the GEOMOETRY column from the long / lat
UPDATE airports SET location = ST_GeomFromText('POINT('||longitude_deg||' '|| latitude_deg||')',4326);

-- TRUNCATE TABLE airports;
-- DROP TABLE airports;
-- SELECT * FROM airports LIMIT 10;
-- SELECT to_char(count(*), '999G999') FROM airports;

-- Runways
CREATE TABLE runways (
  id BIGINT PRIMARY KEY,
  airport_ref INT,
  airport_ident CHAR(10) REFERENCES airports(ident),
  length_ft INT,
  width_ft INT,
  surface VARCHAR(200),
  lighted BOOLEAN,
  closed BOOLEAN,
  le_ident VARCHAR(10),
  le_latitude_deg REAL,
  le_longitude_deg REAL,
  le_elevation_ft INT,
  le_heading_deg REAL,
  le_displaced_threshold_ft INT,
  he_ident CHAR(20),
  he_latitude_deg REAL,
  he_longitude_deg REAL,
  he_elevation_ft INT,
  he_heading_deg REAL,
  he_displaced_threshold_ft INT);

COMMENT ON TABLE runways IS 'Data from http://ourairports.com/data/';

COPY runways FROM '/Pgcon14/Scripts/020_Airports/runways.csv' WITH DELIMITER ',' CSV HEADER;

-- TRUNCATE TABLE runways;
-- DROP TABLE runways;
-- SELECT * FROM runways LIMIT 10;
-- SELECT to_char(count(*), '999G999') FROM runways;

-- Regions
CREATE TABLE regions (
  id BIGINT PRIMARY KEY,
  code VARCHAR(10),
  local_code VARCHAR(10),
  name VARCHAR(100),
  continent CHAR(2),
  iso_country CHAR(2) REFERENCES countries(code),
  wikipedia_link VARCHAR(500),
  keywords VARCHAR(100));

COMMENT ON TABLE regions IS 'Data from http://ourairports.com/data/';

COPY regions FROM '/Pgcon14/Scripts/020_Airports/regions.csv' WITH DELIMITER ',' CSV HEADER;

-- TRUNCATE TABLE regions;
-- DROP TABLE regions;
-- SELECT * FROM regions LIMIT 10;
-- SELECT to_char(count(*), '9G999') FROM regions;

-- How many records do we have
SELECT 'airports' AS table, to_char(count(*), '999G999') AS num_records FROM airports
UNION
SELECT 'countries' AS table, to_char(count(*), '999G999') AS num_records FROM countries
UNION
SELECT 'regions' AS table, to_char(count(*), '999G999') AS num_records FROM regions
UNION
SELECT 'runways' AS table, to_char(count(*), '999G999') AS num_records FROM runways;
