﻿-- pgCon2014 PostgreSQL Working with Network Data
-- James Hanson 21-May-14
-- Sample Apache access log
-- Downloaded from http://notebook.cowgar.com/access.log
--      Search for other files in Google with "inurl:access.log filetype:log"
-- NOTE:  I removed the rows listed in access.log.removed to simplify processing.
-- NOTE2: The source file is "live" and continues to grow

-- COPY into temporary table for processing
-- Create temporary table
CREATE TABLE apache_log_temp (
   id SERIAL,
   col1 TEXT);

-- Set the client_encoding for the same reason as before
SET client_encoding TO 'latin1';

-- Load temporary table
COPY apache_log_temp(  
  col1)
 FROM '/Pgcon14/Scripts/080_ApacheLogs/access.log'
 WITH DELIMITER '^'  -- use a character that is not in the dataset
 -- CSV
 ;

-- Removed lines listed in access.log.removed

-- TRUNCATE TABLE apache_log_temp;
-- DROP TABLE apache_log_temp;
-- SELECT * FROM apache_log_temp LIMIT 100;

-- Pull the fields we want
-- regexp_replace [, ], ' /'
WITH apache_log_array AS (
SELECT 
  id,
  regexp_split_to_array(col1, ' ') AS col1
FROM apache_log_temp
LIMIT 53369
)
SELECT 
  id,
  col1[1] AS col1_1,
  col1[2] AS col1_2,
  col1[3] AS col1_3,
  col1[4] AS col1_4,
  col1[5] AS col1_5,
  col1[6] AS col1_6,
  col1[7] AS col1_7,
  col1[8] AS col1_8,
  col1[9] AS col1_9,
  col1[10] AS col1_10,
  col1[11] AS col1_11,
  col1[12] AS col1_12,
  col1[13] AS col1_13,
  col1[14] AS col1_14,
  col1[15] AS col1_15,
  col1[16] AS col1_16,
  col1[17] AS col1_17,
  col1[18] AS col1_18,
  col1[19] AS col1_19  
FROM apache_log_array
ORDER BY id DESC;

-- Manipulate and CAST the fields as needed
-- got the column definitions from a Excel file
-- Pull the fields we want
-- regexp_replace [, ], ' /'
WITH apache_log_array AS (
SELECT 
  id,
  regexp_split_to_array(col1, ' ') AS col1
FROM apache_log_temp
-- LIMIT 5000
)
SELECT 
  id,
  col1[1]::inet AS requesting_ip,
  to_timestamp(RIGHT(col1[4], -1) || LEFT(col1[5], -6), 'DD/Mon/YYYY:HH24:MI:SS') AT TIME ZONE INTERVAL '-04:00' AS time,
  RIGHT(col1[6], -1)::VARCHAR(10) AS action,
  col1[7]::VARCHAR(100) AS resource,
  LEFT(col1[8], -1)::VARCHAR(10) AS protocol,
  NULLIF(col1[9], '-')::bigint AS status_to_client,
  NULLIF(col1[10], '-')::bigint AS object_size_bytes,
  REPLACE(NULLIF(col1[11], '"-"'), '"', '')::VARCHAR(100) AS referrer,
  REPLACE(COALESCE(col1[12], '') || COALESCE(col1[13], '') || COALESCE(col1[14], '') || COALESCE(col1[15], '') || COALESCE(col1[16], '') || COALESCE(col1[17], '') 
    || COALESCE(col1[18], '') || COALESCE(col1[19], '') || COALESCE(col1[20], ''), '"', '')::VARCHAR(200) AS user_agent
FROM apache_log_array
ORDER BY id DESC;

-- Try it once with a LIMIT statement
SELECT *
INTO apache_log
FROM (
WITH apache_log_array AS (
SELECT 
  regexp_split_to_array(col1, ' ') AS col1
FROM apache_log_temp
-- LIMIT 1000
)
SELECT 
  col1[1]::inet AS requesting_ip,
  to_timestamp(RIGHT(col1[4], -1) || LEFT(col1[5], -6), 'DD/Mon/YYYY:HH24:MI:SS') AT TIME ZONE INTERVAL '-04:00' AS time,
  RIGHT(col1[6], -1)::VARCHAR(10) AS action,
  col1[7]::VARCHAR(100) AS resource,
  LEFT(col1[8], -1)::VARCHAR(10) AS protocol,
  NULLIF(col1[9], '-')::int AS status_to_client,
  NULLIF(col1[10], '-')::int AS object_size_bytes,
  REPLACE(NULLIF(col1[11], '"-"'), '"', '')::VARCHAR(100) AS referrer,
  REPLACE(COALESCE(col1[12], '') || COALESCE(col1[13], '') || COALESCE(col1[14], '') || COALESCE(col1[15], '') || COALESCE(col1[16], '') || COALESCE(col1[17], '') 
    || COALESCE(col1[18], '') || COALESCE(col1[19], '') || COALESCE(col1[20], ''), '"', '')::VARCHAR(200) AS user_agent
FROM apache_log_array
) AS tmp_table;

COMMENT ON TABLE apache_log IS 'Downloaded from http://notebook.cowgar.com/access.log';

-- TRUNCATE TABLE apache_log;
-- DROP TABLE apache_log;
-- SELECT * FROM apache_log LIMIT 100;
-- SELECT to_char(count(*), '999G999') FROm apache_log;

-- Cleanup
DROP TABLE apache_log_temp;
SET client_encoding TO 'UTF-8';