﻿-- pgCon2014 PostgreSQL and Network Data workshop
-- James Hanson 21-May-14
-- CAIDA (Cooperative Association for Internet Data Analysis)
-- See http://www.caida.org/data/overview/
-- Downloaded from http://data.caida.org/datasets/topology/ark/ipv4/dns-names/2012/03/dns-names.l7.20120301.txt.gz
--                 http://data.caida.org/datasets/topology/ark/ipv4/dns-names/2012/03/dns-names.l7.20120302.txt.gz
--                 http://data.caida.org/datasets/topology/ark/ipv4/dns-names/2012/03/dns-names.l7.20120303.txt.gz
--                 http://data.caida.org/datasets/topology/ark/ipv4/dns-names/2012/03/dns-names.l7.20120304.txt.gz
--


-- COPY into temporary table for processing because file contains header and footer lines
--    that the COPY command cannot handle
-- Create temporary table
-- Load DNS file 1
CREATE TABLE dns1_temp (
   id SERIAL,
   col1 TEXT);

-- Load temporary table
-- NOTE:  I did not use a straight COPY command because we have to convert the time from epoch to timestamp
COPY dns1_temp(  
  col1)
 FROM '/Pgcon14/Scripts/100_CAIDA/dns-names.l7.20120301.txt'
 WITH DELIMITER '$'   -- use a character that is not in the dataset
 CSV;

-- TRUNCATE TABLE dns1_temp;
-- DROP TABLE dns1_temp;
-- SELECT * FROM dns1_temp LIMIT 100;
-- SELECT to_char(count(*), '999G999G999') FROM dns1_temp;

-- Use regexp_split_to_array to look at the data
WITH dns1_array AS (
SELECT regexp_split_to_array(col1, '\t') AS col1
FROM dns1_temp
LIMIT 10
)
SELECT 
  col1[1] AS col1_1,
  col1[2] AS col1_2,
  col1[3] AS col1_3,
  col1[4] AS col1_4
FROM dns1_array;

-- Manipulate and CAST the fields as needed
WITH dns1_array AS (
SELECT regexp_split_to_array(col1, '\t') AS col1
FROM dns1_temp
LIMIT 100
)
SELECT 
  to_timestamp(col1[1]::bigint) AS dns_timestamp,
  col1[2]::inet AS ip,
  col1[3]::VARCHAR(200) AS name,
--  col1[4] AS col1_4
  'dns-names.l7.20120301.txt'::CHAR(25) AS file_source
FROM dns1_array;

-- Create the dns table with data from the first data file
SELECT *
INTO dns
FROM (
WITH dns1_array AS (
SELECT regexp_split_to_array(col1, '\t') AS col1
FROM dns1_temp
-- LIMIT 100
)
SELECT 
  to_timestamp(col1[1]::bigint) AS dns_timestamp,
  col1[2]::inet AS ip,
  col1[3]::VARCHAR(200) AS name,
--  col1[4] AS col1_4
  'dns-names.l7.20120301.txt'::CHAR(25) AS file_source
FROM dns1_array
) AS tmp_table;

-- TRUNCATE TABLE dns;
-- DROP TABLE dns;
-- SELECT * FROM dns LIMIT 100;
-- SELECT to_char(count(*), '999G999G999') FROM dns;

-- Load DNS files  2 - 4
CREATE TABLE dns2_temp (
   id SERIAL,
   col1 TEXT);

CREATE TABLE dns3_temp (
   id SERIAL,
   col1 TEXT);

CREATE TABLE dns4_temp (
   id SERIAL,
   col1 TEXT);  

-- Load temporary tables 2, 3 and 4
-- NOTE:  I did not use a straight COPY command because we have to convert the time from epoch to timestamp
COPY dns2_temp(  
  col1)
  FROM '/Pgcon14/Scripts/100_CAIDA/dns-names.l7.20120302.txt'
  WITH DELIMITER '$'   -- use a character that is not in the dataset
  CSV;
-- SELECT to_char(count(*), '999G999G999') FROM dns2_temp;

COPY dns3_temp(  
  col1)
  FROM '/Pgcon14/Scripts/100_CAIDA/dns-names.l7.20120303.txt'
  WITH DELIMITER '$'   -- use a character that is not in the dataset
  CSV;
-- SELECT to_char(count(*), '999G999G999') FROM dns3_temp;

COPY dns4_temp(  
  col1)
  FROM '/Pgcon14/Scripts/100_CAIDA/dns-names.l7.20120304.txt'
  WITH DELIMITER '$'   -- use a character that is not in the dataset
  CSV;  
-- SELECT to_char(count(*), '999G999G999') FROM dns4_temp;

-- Note change in syntax to INSERT INTO an existing table
-- dns2
INSERT INTO dns 
SELECT *
FROM (
WITH dns2_array AS (
SELECT regexp_split_to_array(col1, '\t') AS col1
FROM dns2_temp
-- LIMIT 100
)
SELECT 
  to_timestamp(col1[1]::bigint) AS dns_timestamp,
  col1[2]::inet AS ip,
  col1[3]::VARCHAR(200) AS name,
--  col1[4] AS col1_4
  'dns-names.l7.20120302.txt'::CHAR(25) AS file_source
FROM dns2_array
) AS tmp_table;

-- dns3
INSERT INTO dns 
SELECT *
FROM (
WITH dns3_array AS (
SELECT regexp_split_to_array(col1, '\t') AS col1
FROM dns3_temp
-- LIMIT 100
)
SELECT 
  to_timestamp(col1[1]::bigint) AS dns_timestamp,
  col1[2]::inet AS ip,
  col1[3]::VARCHAR(200) AS name,
--  col1[4] AS col1_4
  'dns-names.l7.20120303.txt'::CHAR(25) AS file_source
FROM dns3_array
) AS tmp_table;

-- dns4
INSERT INTO dns 
SELECT *
FROM (
WITH dns4_array AS (
SELECT regexp_split_to_array(col1, '\t') AS col1
FROM dns4_temp
-- LIMIT 100
)
SELECT 
  to_timestamp(col1[1]::bigint) AS dns_timestamp,
  col1[2]::inet AS ip,
  col1[3]::VARCHAR(200) AS name,
--  col1[4] AS col1_4
  'dns-names.l7.20120304.txt'::CHAR(25) AS file_source
FROM dns4_array
) AS tmp_table;

COMMENT ON TABLE dns IS 'Downloaded from http://data.caida.org/datasets/topology/ark/ipv4/dns-names/2012/03/dns-names.l7.2012030[1,2,3,4].txt.gz';

SELECT 
  file_source,
  to_char(count(*), '999G999G999')
FROM dns
GROUP BY file_source
ORDER BY file_source;

-- TRUNCATE TABLE dns;
-- DROP TABLE dns;
-- SELECT * FROM dns LIMIT 100;
-- SELECT to_char(count(*), '999G999G999') FROM dns;

-- cleanup
DROP TABLE dns1_temp;
DROP TABLE dns2_temp;
DROP TABLE dns3_temp;
DROP TABLE dns4_temp;

