﻿-- pgCon2014 PostgreSQL Working with Network Data
-- James Hanson 21-May-14
-- IPv6 GeoLocation
-- Downloaded from http://geolite.maxmind.com/download/geoip/database/GeoLiteCityv6-beta/GeoLiteCityv6.csv.gz

-- COPY into temporary table for processing because file contains header and footer lines
--    that the COPY command cannot handle
-- Create temporary table
CREATE TABLE ipv6_geo_temp (
   id SERIAL,
   col1 TEXT);

-- Load temporary table
COPY ipv6_geo_temp(  
  col1)
 FROM '/Pgcon14/Scripts/060_IPv6GeoLoc/GeoLiteCityv6.csv'
 WITH DELIMITER '$'   -- use a character that is not in the dataset
 CSV;

-- TRUNCATE TABLE ipv6_geo_temp;
-- DROP TABLE ipv6_geo_temp;
-- SELECT * FROM ipv6_geo_temp LIMIT 100;

-- Use regexp_split_to_array to look at the data
WITH ipv6_geo_array AS (
SELECT 
  id,
  regexp_split_to_array(col1, '\,') AS col1
FROM ipv6_geo_temp
LIMIT 100
)
SELECT 
  id,
  col1[1] AS col1_1,
  col1[2] AS col1_2,
  col1[3] AS col1_3,
  col1[4] AS col1_4,
  col1[5] AS col1_5,
  col1[6] AS col1_6,
  col1[7] AS col1_7,
  col1[8] AS col1_8,
  col1[9] AS col1_9,
  col1[10] AS col1_10,
  col1[11] AS col1_11,
  col1[12] AS col1_12,
  col1[13] AS col1_13
FROM ipv6_geo_array;

-- Manipulate and CAST the fields as needed
-- got the column definitions from a blog post
WITH ipv6_geo_array AS (
SELECT 
  id,
  regexp_split_to_array(col1, '\,') AS col1
FROM ipv6_geo_temp
-- LIMIT 1000
)
SELECT 
  id,
  col1[1]::inet AS startIP,
  col1[2]::inet AS endIP,
--  col1[3] AS col1_3, -- IP numeric ... for people who don't have PostgreSQL inet type!
--  col1[4] AS col1_4, -- IP numeric ... for people who don't have PostgreSQL inet type!
  col1[5]::CHAR(2) AS country_code,
  col1[6]::CHAR(2) AS region_code,
--  col1[7] AS col1_8,
  col1[8]::real AS latitude,
  col1[9]::real AS longitude,
--  col1[10] AS col1_10,
--  col1[11] AS col1_11,
--  col1[12] AS col1_12,
--  col1[13] AS col1_13
  ST_GeomFromText('POINT('||col1[9]::real||' '|| col1[8]::real||')',4326) AS location
FROM ipv6_geo_array;

-- Create and populate ipv6_geo from ipv6_geo_temp
-- Try it once with a LIMIT statement
SELECT * 
INTO ipv6_geo
FROM (
WITH ipv6_geo_array AS (
SELECT 
  id,
  regexp_split_to_array(col1, '\,') AS col1
FROM ipv6_geo_temp
-- LIMIT 1000
)
SELECT 
  id,
  col1[1]::inet AS startIP,
  col1[2]::inet AS endIP,
--  col1[3] AS col1_3, -- IP numeric ... for people who don't have PostgreSQL inet type!
--  col1[4] AS col1_4, -- IP numeric ... for people who don't have PostgreSQL inet type!
  col1[5]::CHAR(2) AS country_code,
  col1[6]::CHAR(2) AS region_code,
--  col1[7] AS col1_8,
  col1[8]::real AS latitude,
  col1[9]::real AS longitude,
--  col1[10] AS col1_10,
--  col1[11] AS col1_11,
--  col1[12] AS col1_12,
--  col1[13] AS col1_13
  ST_GeomFromText('POINT('||col1[9]::real||' '|| col1[8]::real||')',4326) AS location
FROM ipv6_geo_array
) AS tmp_table;

COMMENT ON TABLE ipv6_geo IS 'http://geolite.maxmind.com/download/geoip/database/GeoLiteCityv6-beta/GeoLiteCityv6.csv.gz';

-- TRUNCATE TABLE ipv6_geo;
-- DROP TABLE ipv6_geo;
-- SELECT * FROM ipv6_geo LIMIT 100;
-- SELECT to_char(count(*), '999G999') AS num_recs FROM ipv6_geo;

-- NOTE:  We could create an inettype range for the startip and endip.  
--        We'll do that with other data sets ... but I couldn't find any IPv6 data to validate against it

-- cleanup
DROP TABLE ipv6_geo_temp;

-- We just loaded 18K international IPv6 GeoLocation records - 
-- including validation and tru GeoSpatial - in ~2min.
--   That's pretty cool!
