﻿-- pgCon2014 Working with Network Data
-- James Hanson 21-May-14
-- US IPGeo
-- Downloaded from http://infochimps.org/datasets/united-states-ip-address-to-geolocation-data 

-- Were stuck with a 2-step loading process because of the GEOMETRY data type and extra fields
--   ... so let's try something different.

-- COPY into temporary table for processing
-- Create temporary table
CREATE TABLE us_ipgeo_temp (
   id SERIAL,
   col1 TEXT);

-- Load temporary table
COPY us_ipgeo_temp(  
  col1)
 FROM '/Pgcon14/Scripts/030_USIPGeo/infochimps_dataset_13358_download_16680/ip_blocks_us_geo.tsv'
 WITH DELIMITER '|'  -- use a character that is not in the dataset
 CSV;

-- TRUNCATE TABLE us_ipgeo_temp;
-- DROP TABLE us_ipgeo_temp;
-- SELECT * FROM us_ipgeo_temp LIMIT 100;
-- SELECT to_char(count(*), '999G999G999') FROM us_ipgeo_temp;

-- Use a CTE (Common Table Expression) syntax and
--    regexp_split_to_array to look at the data in our temporary table.
WITH us_ipgeo_array AS (
SELECT regexp_split_to_array(col1, '\t') AS col1
FROM us_ipgeo_temp
LIMIT 10)
SELECT 
  col1[1] AS col1_1,
  col1[2] AS col1_2,
  col1[3] AS col1_3,
  col1[4] AS col1_4,
  col1[5] AS col1_5,
  col1[6] AS col1_6,
  col1[7] AS col1_7,
  col1[8] AS col1_8,
  col1[9] AS col1_9,
  col1[10] AS col1_10,
  col1[11] AS col1_11,
  col1[12] AS col1_12,
  col1[13] AS col1_13,
  col1[14] AS col1_14
FROM us_ipgeo_array;

-- Pull the fields we want
WITH us_ipgeo_array AS (
SELECT regexp_split_to_array(col1, '\t') AS col1
FROM us_ipgeo_temp
LIMIT 100)
SELECT 
  -- col1[1] AS col1_1,
  -- col1[2] AS col1_2,
  -- col1[3] AS col1_3,
  -- col1[4] AS col1_4,
  col1[5] AS col1_5,
  col1[6] AS col1_6,
  col1[7] AS col1_7,
  col1[8] AS col1_8,
  col1[9] AS col1_9,
  col1[10] AS col1_10,
  col1[11] AS col1_11,
  col1[12] AS col1_12,
  col1[13] AS col1_13
  -- col1[14] AS col1_14
FROM us_ipgeo_array
ORDER BY col1_12;

-- Manipulate and CAST the fields as needed
WITH us_ipgeo_array AS (
SELECT regexp_split_to_array(col1, '\t') AS col1
FROM us_ipgeo_temp
LIMIT 10000   -- check more records too
)
SELECT 
  -- col1[1] AS col1_1,
  -- col1[2] AS col1_2,
  -- col1[3] AS col1_3,
  -- col1[4] AS col1_4,
  col1[5]::inet  AS ip,
  REPLACE(col1[6], '"', '')::CHAR(2) AS country,
  REPLACE(col1[7], '"', '')::CHAR(2) AS state,
  REPLACE(col1[8], '"', '')::VARCHAR(80) AS city,
  REPLACE(col1[9], '"', '')::CHAR(5) AS zip_code,
  col1[10]::real AS latitude,
  col1[11]::real AS longitude,
  ST_GeomFromText('POINT('||col1[11]::real||' '|| col1[10]::real||')',4326) AS location,
  NULLIF(col1[12], '')::int AS metro_code,  -- A few values were empty string, IS NOT NULL and does not CAST to INT
  col1[13]::text AS area_code
  -- col1[14] AS col1_14
FROM us_ipgeo_array
ORDER BY metro_code NULLS LAST;

-- Create and populate us_ipgeo from us_ipgeo_temp
-- Try it once with a LIMIT statement
SELECT * 
INTO us_ipgeo
FROM (
WITH us_ipgeo_array AS (
SELECT regexp_split_to_array(col1, '\t') AS col1
FROM us_ipgeo_temp
-- LIMIT 10000   -- check more records too
)
SELECT 
  -- col1[1] AS col1_1,
  -- col1[2] AS col1_2,
  -- col1[3] AS col1_3,
  -- col1[4] AS col1_4,
  col1[5]::inet  AS ip,
  REPLACE(col1[6], '"', '')::CHAR(2) AS country,
  REPLACE(col1[7], '"', '')::CHAR(2) AS state,
  REPLACE(col1[8], '"', '')::VARCHAR(80) AS city,
  REPLACE(col1[9], '"', '')::CHAR(5) AS zip_code,
  col1[10]::real AS latitude,
  col1[11]::real AS longitude,
  ST_GeomFromText('POINT('||col1[11]::real||' '|| col1[10]::real||')',4326) AS location,
  NULLIF(col1[12], '')::int AS metro_code,  -- A few values were empty string, IS NOT NULL and does not CAST to INT
  col1[13]::text AS area_code
  -- col1[14] AS col1_14
FROM us_ipgeo_array
) AS tmp_table;

COMMENT ON TABLE us_ipgeo IS 'Downloaded from http://infochimps.org/datasets/united-states-ip-address-to-geolocation-data';

-- TRUNCATE TABLE us_ipgeo;
-- DROP TABLE us_ipgeo;
-- SELECT * FROM us_ipgeo LIMIT 100;
-- SELECT to_char(count(*), '999G999G999') FROM us_ipgeo LIMIT 100;

-- Check the results
SELECT * 
FROM us_ipgeo
WHERE state = 'MD'
AND city = 'Columbia'
LIMIT 100;

-- Cleanup
DROP TABLE us_ipgeo_temp;

-- We loaded 1.4 million IPGeo records with true GeoSpatial values in ~5 min.  That's cool!!!