﻿-- pgCon2014 Working with Netowork Data
-- James Hanson 21-May-14
-- ARP record data 
-- Downloaded from "http://wiki.nil.com/ARP_table_display"

-- COPY into temporary table for processing because file contains header and footer lines
--    that the COPY command cannot handle
-- Create temporary table
CREATE TABLE arp_temp (
   id SERIAL,
   col1 TEXT);

-- Load temporary table
COPY arp_temp(  
  col1)
 FROM '/Pgcon14/Scripts/120_ARP/nil_arp.txt'
 WITH DELIMITER '$'   -- use a character that is not in the dataset
 CSV;

-- TRUNCATE TABLE arp_temp;
-- DROP TABLE arp_temp;
-- SELECT count(*) arp_temp;

-- remove 2 header rows
DELETE FROM arp_temp
WHERE id = any(array(
  SELECT id
  FROM arp_temp
  ORDER BY id
  LIMIT 2)
  );

-- Use regexp_split_to_array to look at the data
WITH arp_array AS (
SELECT regexp_split_to_array(col1, ' +') AS col1 -- treat consecutive delimiters as one
FROM arp_temp
LIMIT 10
)
SELECT 
  col1[1] AS col1_1,
  col1[2] AS col1_2,
  col1[3] AS col1_3,
  col1[4] AS col1_4,
  col1[5] AS col1_5
FROM arp_array;
-- There is a problem with the NULL (missing) value in the 1st tuple

-- Use replace to put 'NULL' in place of the missing value
UPDATE arp_temp
SET col1 = replace(col1, repeat(' ',19), ' NULL ');

-- Now it looks good
WITH arp_array AS (
SELECT regexp_split_to_array(col1, ' +') AS col1 -- treat consecutive delimiters as one
FROM arp_temp
LIMIT 10
)
SELECT 
  col1[1] AS col1_1,
  col1[2] AS col1_2,
  col1[3] AS col1_3,
  col1[4] AS col1_4,  
  col1[5] AS col1_5
FROM arp_array;

-- Manipulate and CAST the fields as needed
WITH arp_array AS (
SELECT regexp_split_to_array(col1, ' +') AS col1 -- treat consecutive delimiters as one
FROM arp_temp
LIMIT 10
)
SELECT 
  col1[1]::inet AS IP_address,
  col1[2]::macaddr AS MAC_address,
  NULLIF(col1[3]::VARCHAR(25), 'NULL') AS Physical_intf,  -- Change the string to an actual NULL
  col1[4]::VARCHAR(25) AS Logical_intf
FROM arp_array;

-- Create the ARP table
SELECT * 
INTO arp
FROM (
WITH arp_array AS (
SELECT regexp_split_to_array(col1, ' +') AS col1 -- treat consecutive delimiters as one
FROM arp_temp
-- LIMIT 10
)
SELECT 
  col1[1]::inet AS IP_address,
  col1[2]::macaddr AS MAC_address,
  NULLIF(col1[3]::VARCHAR(25), 'NULL') AS Physical_intf,  -- Change the string to an actual NULL
  col1[4]::VARCHAR(25) AS Logical_intf
FROM arp_array
) AS tmp_table;

COMMENT ON TABLE arp IS 'Downloaded from http://wiki.nil.com/ARP_table_display';

-- TRUNCATE TABLE arp;
-- DROP TABLE arp;
-- SELECT * FROM arp;

-- cleanup
DROP TABLE arp_temp;

-- Now let's play a bit ...

-- Create a custom type for the physical interface
-- DROP TYPE p_intf;
CREATE TYPE p_intf AS (
  ip_address INET,
  mac_address MACADDR
);

-- SELECT from arp creating an array
SELECT
  physical_intf,
  ARRAY_AGG((ip_address, mac_address)::p_intf) AS pintf_arr,
  logical_intf
FROM arp
GROUP BY
  physical_intf,
  logical_intf;

SELECT *
INTO arp_arr
FROM (
SELECT
  physical_intf,
  ARRAY_AGG((ip_address, mac_address)::p_intf) AS pintf_arr,
  logical_intf
FROM arp
GROUP BY
  physical_intf,
  logical_intf
 ) temp_table;

-- SELECT * FROM arp_arr;
-- DROP TABLE arp_arr;

-- This enables us to select from the custom array using the ANY operator
SELECT *
FROM arp_arr
WHERE ('192.168.200.196', '00:16:41:e6:dc:2c')::p_intf = ANY(pintf_arr);
