﻿-- pgCon2014 PostgreSQL and Network Data workshop
-- James Hanson 21-May-14
-- tracert data DDL statements
-- Downloaded from http://ita.ee.lbl.gov/html/contrib/NPD-Routes.html
--     and pre-processed using tracert_etl.py

-- tracert
CREATE TABLE tracert (
  id SERIAL,
  source_file VARCHAR(100),
  dest_name VARCHAR(100),
  dest_ip inet,
  hop INT,
  from_ip inet,
  to_ip inet);
  
COMMENT ON TABLE tracert IS 'Downloaded from http://ita.ee.lbl.gov/html/contrib/NPD-Routes.html';

COPY tracert (source_file, dest_name, dest_ip, hop, from_ip, to_ip)
  FROM '/Pgcon14/Scripts/090_tracert/tracert.csv' WITH DELIMITER E'\t' CSV HEADER;
 
-- TRUNCATE TABLE tracert;
-- DROP TABLE tracert;
-- SELECT * FROM tracert LIMIT 10;
-- SELECT to_char(COUNT(*), '999G999') AS num_recs FROM tracert;


-- Lets play a bit with an array structure for the tracert table
-- DROP TYPE tracert_trip;

CREATE TYPE tracert_trip AS (
  hop INTEGER,
  from_ip INET,
  to_ip INET
);


SELECT 
  source_file,
  dest_name,
  dest_ip,
  array_agg((hop, from_ip, to_ip)::tracert_trip ORDER BY hop) tracert_trip
FROM tracert
GROUP BY
   source_file,
   dest_name,
   dest_ip
LIMIT 10000;

-- Look at the tracert_trip column

CREATE TABLE tracert_array AS (
SELECT 
  source_file,
  dest_name,
  dest_ip,
  array_agg((hop, from_ip, to_ip)::tracert_trip ORDER BY hop) tracert_trip
FROM tracert
-- WHERE dest_ip = '129.12.3.53' 
GROUP BY
   source_file,
   dest_name,
   dest_ip
-- LIMIT 10000
);

-- Take a look at the tracert_array
SELECT *
FROM tracert_array
LIMIT 100;

SELECT
  dest_name,
  tracert_trip[1].hop,
  tracert_trip[1].from_ip,
  tracert_trip[1].to_ip
FROM tracert_array;
  
-- How much smaller is tracert_aray than tracert?
-- Rows in tracert
SELECT TO_CHAR(count(*), '999G999G999') num_recs
FROM tracert;

-- Rows in tracert_array
SELECT TO_CHAR(count(*), '999G999G999') num_recs
FROM tracert_array;

-- How much smaller is tracert_array than tracert?
SELECT  
  ROUND(
    (SELECT count(*) FROM tracert) /
    (SELECT count(*) FROM tracert_array)::NUMERIC,
  1)::text || ' times smaller';


